package gdxforgelauncher;

import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextField;
import javafx.stage.Stage;
import javafx.stage.WindowEvent;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

public class Controller implements Initializable {
    public Button btnQuit;
    public Button btnLaunchForge;
    public TextField txtWidth;
    public TextField txtHeight;
    public CheckBox chkFullscreen;

    private Stage stage;
    public void setStage(Stage stage0) {
        stage = stage0;
    }

    private int width = 1920, height = 1080;
    private boolean fullscreen = true;
    private double scaleFactor = 1.0F;

    public int getWidth() {
        if (Utils.isNumeric(txtWidth.getText())) {
            width = Integer.valueOf(txtWidth.getText());
        }
        return width;
    }
    public int getHeight() {
        if (Utils.isNumeric(txtHeight.getText())) {
            height = Integer.valueOf(txtHeight.getText());
        }
        return height;
    }
    public boolean getFullscreen() {
        fullscreen = chkFullscreen.isSelected();
        return fullscreen;
    }
    public double getScaleFactor() {
        scaleFactor = stage.getWidth() / 440;
        return scaleFactor;
    }

    @Override
    public void initialize(URL url, ResourceBundle resource) {
        if (Utils.doesFileExist("screen_resolution.ini")) {
            String[] res = Utils.readFileToString("screen_resolution.ini").split("x");
            if (Utils.isNumeric(res[0]) && Utils.isNumeric(res[1]) && Utils.isNumeric(res[2])) {
                fullscreen = res.length == 3 ? Integer.parseInt(res[2].trim()) > 0 : true;
                if (res.length >= 2) {
                    width = Integer.parseInt(res[0].trim());
                    height = Integer.parseInt(res[1].trim());
                }
            }
        }

        txtWidth.setText(String.valueOf(width));
        txtHeight.setText(String.valueOf(height));
        chkFullscreen.setSelected(fullscreen);
    }

    public static void saveSettings(int width, int height, boolean fullscreen, double scaleFactor) {
        StringBuilder resDef = new StringBuilder();
        resDef.append(String.valueOf(width));
        resDef.append("x");
        resDef.append(String.valueOf(height));
        resDef.append("x");
        resDef.append(fullscreen ? "1" : "0");
        Utils.writeFile(new File("screen_resolution.ini"), resDef.toString());

        Utils.writeFile(new File("MobileForgeLauncher.ini"), String.valueOf(scaleFactor));
    }

    public void btnQuitClick(ActionEvent actionEvent) {
        saveSettings(getWidth(), getHeight(), getFullscreen(), getScaleFactor());
        Platform.exit();
        System.exit(0);
    }

    public void btnLaunchForgeClick(ActionEvent actionEvent) {
        File forgeJar = Utils.getForgeExecutable();
        boolean exists = forgeJar != null, launched = exists;

        if (!exists) {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Error launching Forge");
            alert.setHeaderText("Could not launch Forge GDX");
            alert.setContentText("Game executable JAR file not found.");
            alert.showAndWait();
            return;
        }

        // set up and save resolution settings
        if (!Utils.isNumeric(txtWidth.getText()) || !Utils.isNumeric(txtHeight.getText())) {
            txtWidth.setText(String.valueOf(width));
            txtHeight.setText(String.valueOf(height));
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Error launching Forge");
            alert.setHeaderText("Could not launch Forge GDX");
            alert.setContentText("Resolution settings are incorrect,\nlast known values restored.");
            alert.showAndWait();
            return;
        }
        saveSettings(getWidth(), getHeight(), getFullscreen(), getScaleFactor());

        // detect 32bit/64bit to pass the appropriate memory request later
        boolean is64BitJVM = System.getProperty("sun.arch.data.model").equals("64");

        if (System.getProperty("sun.arch.data.model").equals("32")) {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Warning");
            alert.setHeaderText("64-bit Java Recommended");
            alert.setContentText("You are attempting to run the game on a 32-bit\ninstallation of Java. While this might work,\nyou may run out of memory. Please consider\ninstalling 64-bit Java Runtime Environment.");
            alert.showAndWait();
        }

        // launch the main process, assuming the file is in the current (root) folder
        ProcessBuilder pb = new ProcessBuilder("java", is64BitJVM ? "-Xmx1536m" : "-Xmx1024m", "-jar", forgeJar.getName());
        try {
            pb.inheritIO().start();
        } catch (IOException e) {
            e.printStackTrace();
            launched = false;
        }

        if (!launched) {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Error launching Forge");
            alert.setHeaderText("Could not launch Forge GDX");
            alert.setContentText("Please check that the game is installed correctly.");
            alert.showAndWait();
            return;
        }

        Platform.exit();
        System.exit(0);
    }
}
